@@ -1,16 +1,17 @@
-# SPDX-FileCopyrightText: 2022 Dan Halbert for Adafruit Industries
+# SPDX-FileCopyrightText: 2023 Michał Pokusa
 #
 # SPDX-License-Identifier: Unlicense
 
 import socketpool
 import wifi
 
-from adafruit_httpserver import Server, Request, Response, Basic, Bearer
+from adafruit_httpserver import Server, Request, Response, Basic, Token, Bearer
 
 
 # Create a list of available authentication methods.
 auths = [
     Basic("user", "password"),
+    Token("2db53340-4f9c-4f70-9037-d25bee77eca6"),
     Bearer("642ec696-2a79-4d60-be3a-7c9a3164d766"),
 ]
 
